# !/usr/bin/env python
# encoding: utf-8

"""
Generate the code reference pages and navigation.

Adapted from https://mkdocstrings.github.io/recipes.
"""

from pathlib import Path

import mkdocs_gen_files

nav = mkdocs_gen_files.Nav()

root = 'ckanext'

py_files = sorted(Path(root).rglob('*.py'))

for path in py_files:
    module_path = path.relative_to(root).with_suffix('')
    doc_path = path.relative_to(root).with_suffix('.md')
    full_doc_path = Path('API', doc_path)

    parts = tuple(module_path.parts)

    if parts[-1] == '__init__':
        if len(parts) == 1:
            continue
        parts = parts[:-1]
        doc_path = doc_path.with_name('index.md')
        full_doc_path = full_doc_path.with_name('index.md')
    elif parts[-1] == '__main__':
        continue

    nav[parts] = doc_path.as_posix()

    with mkdocs_gen_files.open(full_doc_path, "w") as fd:
        ident = '.'.join(parts)
        fd.write(f'::: ckanext.{ident}')

    mkdocs_gen_files.set_edit_path(full_doc_path, path)

with mkdocs_gen_files.open('API/index.md', 'w') as nav_file:
    nav_file.writelines(nav.build_literate_nav())



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of a project
# Created by the Natural History Museum in London, UK

import re

from ckan.plugins import toolkit
from ckanext.tiledmap.lib.utils import get_resource_datastore_fields


def colour_validator(value, context):
    """
    Validate a value is a CSS hex color.

    :param value: the value to validate
    :param context: the context within which this validation is taking place
    :return: the validated value
    """
    if re.match('^#?([0-9a-fA-F]{3}|[0-9a-fA-F]{6})$', value):
        if value[0] != '#':
            return f'#{value}'
        else:
            return value
    else:
        raise toolkit.Invalid(
            toolkit._(
                'Colors must be formed of three or six RGB hex value, '
                'optionally preceded by a # sign (eg. #E55 or #F4A088)'
            )
        )


def float_01_validator(value, context):
    """
    Validates that the value is a float number between 0 and 1.

    :param value: the value
    :param context: the context within which this validation is taking place
    :return: the validated value
    """
    try:
        value = float(value)
    except ValueError:
        raise toolkit.Invalid(toolkit._('Must be a decimal number, between 0 and 1'))
    if value < 0 or value > 1:
        raise toolkit.Invalid(toolkit._('Must be a decimal number, between 0 and 1'))
    return value


def is_datastore_field(value, context):
    """
    Check that the fields are indeed a datastore fields.

    :param value: the value to validate
    :param context: the context within which this validation is taking place
    :return: the value
    """
    passed_fields = value if isinstance(value, list) else [value]
    fields = get_resource_datastore_fields(toolkit.g.resource['id'])
    invalid_fields = [field for field in passed_fields if field not in fields]
    if invalid_fields:
        raise toolkit.Invalid(f'Invalid parameters: {",".join(invalid_fields)}')
    return value


def is_view_id(value, context):
    """
    Ensure this is a view id on the current resource.

    :param value: the value to validate
    :param context: the context within which this validation is taking place
    :return: the value
    """
    if value:
        data = {'id': toolkit.g.resource['id']}
        views = toolkit.get_action('resource_view_list')(context, data)
        if value not in [view['id'] for view in views]:
            raise toolkit.Invalid(toolkit._('Must be a view on the current resource'))
    return value



#!/usr/bin/env python
# encoding: utf-8
#
# This file is part of a project
# Created by the Natural History Museum in London, UK

from ckan.common import json
from ckan.plugins import toolkit, implements, interfaces, SingletonPlugin

from ckanext.tiledmap import routes
from ckanext.tiledmap.config import config as plugin_config
from ckanext.tiledmap.lib import validators
from ckanext.tiledmap.lib.helpers import mustache_wrapper, dwc_field_title
from ckanext.tiledmap.lib.utils import get_resource_datastore_fields

boolean_validator = toolkit.get_validator('boolean_validator')
ignore_empty = toolkit.get_validator('ignore_empty')


class VersionedTiledMapPlugin(SingletonPlugin):
    """
    Map plugin which uses the versioned-datastore-tile-server to render a map of the
    data in a resource.
    """

    implements(interfaces.IConfigurer)
    implements(interfaces.IBlueprint, inherit=True)
    implements(interfaces.ITemplateHelpers)
    implements(interfaces.IResourceView, inherit=True)
    implements(interfaces.IConfigurable)

    # from IConfigurer interface
    def update_config(self, config):
        """
        Add our various resources and template directories to the list of available
        ones.
        """
        toolkit.add_template_directory(config, 'theme/templates')
        toolkit.add_public_directory(config, 'theme/public')
        toolkit.add_resource('theme/assets', 'tiledmap')

    ## IBlueprint
    def get_blueprint(self):
        return routes.blueprints

    # from ITemplateHelpers interface
    def get_helpers(self):
        """
        Add a template helper for formatting mustache templates server side.
        """
        return {'mustache': mustache_wrapper, 'dwc_field_title': dwc_field_title}

    # from IConfigurable interface
    def configure(self, config):
        plugin_config.update(config)

    # from IResourceView interface
    def info(self):
        """
        Return generic info about the plugin.
        """
        return {
            'name': 'versioned_tiledmap',
            'title': 'Map',
            'schema': {
                # plot settings
                'enable_plot_map': [ignore_empty, boolean_validator],
                'plot_point_radius': [ignore_empty, int],
                'plot_point_colour': [ignore_empty, validators.colour_validator],
                'plot_border_width': [ignore_empty, int],
                'plot_border_colour': [ignore_empty, validators.colour_validator],
                # gridded settings
                'enable_grid_map': [ignore_empty, boolean_validator],
                'gridded_grid_resolution': [ignore_empty, int],
                'gridded_cold_colour': [ignore_empty, validators.colour_validator],
                'gridded_hot_colour': [ignore_empty, validators.colour_validator],
                'gridded_range_size': [ignore_empty, int],
                # heatmap settings
                'enable_heat_map': [ignore_empty, boolean_validator],
                'heatmap_point_radius': [ignore_empty, int],
                'heatmap_cold_colour': [ignore_empty, validators.colour_validator],
                'heatmap_hot_colour': [ignore_empty, validators.colour_validator],
                'heatmap_intensity': [ignore_empty, validators.float_01_validator],
                # utfgrid settings
                'enable_utf_grid': [ignore_empty, boolean_validator],
                'utf_grid_title': [ignore_empty, validators.is_datastore_field],
                'utf_grid_fields': [ignore_empty, validators.is_datastore_field],
                # other settings
                'repeat_map': [ignore_empty, boolean_validator],
                'overlapping_records_view': [ignore_empty, validators.is_view_id],
                '__extras': [ignore_empty],
            },
            'icon': 'compass',
            'iframed': True,
            'filterable': True,
            'preview_enabled': False,
            'full_page_edit': False,
        }

    # from IResourceView interface
    def view_template(self, context, data_dict):
        return 'map_view.html'

    # from IResourceView interface
    def form_template(self, context, data_dict):
        return 'map_form.html'

    # from IResourceView interface
    def can_view(self, data_dict):
        """
        Only datastore resources can use this view and they have to have both latitude
        and longitude field names set.
        """
        required_fields = ['datastore_active', '_latitude_field', '_longitude_field']
        return all(data_dict['resource'].get(field, False) for field in required_fields)

    # from IResourceView interface
    def setup_template_variables(self, context, data_dict):
        """
        Setup variables available to templates.
        """
        # TODO: Apply variables to appropriate view
        resource = data_dict['resource']
        resource_view = data_dict['resource_view']
        resource_view_id = resource_view.get('id', None)
        # get the names of the fields on this resource in the datastore
        fields = get_resource_datastore_fields(resource['id'])
        # find all the views on this resource currently
        views = toolkit.get_action('resource_view_list')(
            context, {'id': resource['id']}
        )

        # build a list of view options, adding a default view option of no view first
        view_options = [{'text': toolkit._('(None)'), 'value': ''}]
        # then loop through and add the other views
        for view in views:
            # but make sure we don't add this view to the list of options
            if resource_view_id == view['id']:
                continue
            view_options.append({'text': view['title'], 'value': view['id']})

        return {
            'resource_json': json.dumps(resource),
            'resource_view_json': json.dumps(resource_view),
            'map_fields': [{'text': field, 'value': field} for field in fields],
            'available_views': view_options,
            'defaults': plugin_config,
            'is_new': resource_view_id is None,
        }



# !/usr/bin/env python
# encoding: utf-8
#
# This file is part of ckanext-versioned-tiledmap
# Created by the Natural History Museum in London, UK

from flask import Blueprint, jsonify

from . import _helpers

blueprint = Blueprint(name='map', import_name=__name__, url_prefix='')


@blueprint.route('/map-info')
def info():
    """
    Returns metadata about a given map in JSON form.

    :return: A JSON encoded string representing the metadata
    """
    view_settings = _helpers.MapViewSettings.from_request()

    # ensure we have at least one map style enabled
    if not view_settings.is_enabled():
        return jsonify({'geospatial': False})

    return jsonify(view_settings.create_map_info())
